@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2020 The Qt Company Ltd.
 ** Copyright (C) 2016 Intel Corporation.
 ** Contact: https://www.qt.io/licensing/
 **
@@ -57,6 +57,7 @@
 #include "QtCore/qvarlengtharray.h"
 #include "QtCore/qvariant.h"
 #include "QtCore/qnumeric.h"
+#include <QtCore/qcalendar.h>
 
 #include "qlocale.h"
 
@@ -86,7 +87,7 @@ public:
         LanguageId, // uint
         CountryId, // uint
         DecimalPoint, // QString
-        GroupSeparator, // QString
+        GroupSeparator, // QString (empty QString means: don't group digits)
         ZeroDigit, // QString
         NegativeSign, // QString
         DateFormatLong, // QString
@@ -113,6 +114,7 @@ public:
         Weekdays, // QList<Qt::DayOfWeek>
         CurrencySymbol, // QString in: CurrencyToStringArgument
         CurrencyToString, // QString in: qlonglong, qulonglong or double
+        Collation, // QString
         UILanguages, // QStringList
         StringToStandardQuotation, // QString in: QStringRef to quote
         StringToAlternateQuotation, // QString in: QStringRef to quote
@@ -170,9 +172,14 @@ Q_DECLARE_TYPEINFO(QLocaleId, Q_PRIMITIVE_TYPE);
 struct QLocaleData
 {
 public:
+    // TODO: Remove this?
     static const QLocaleData *findLocaleData(QLocale::Language language,
                                              QLocale::Script script,
                                              QLocale::Country country);
+    // Having an offset of current locale, enables us to have multiple sources of data, i.e. user-provided calendar locales
+    static uint findLocaleOffset(QLocale::Language language,
+                                 QLocale::Script script,
+                                 QLocale::Country country);
     static const QLocaleData *c();
 
     // Maximum number of significant digits needed to represent a double.
@@ -206,7 +213,8 @@ public:
         ShowBase            = 0x80,
         UppercaseBase       = 0x100,
         ZeroPadExponent     = 0x200,
-        ForcePoint          = 0x400
+        ForcePoint          = 0x400,
+        IndianNumberGrouping= 0x800
     };
 
     enum NumberMode { IntegerMode, DoubleStandardMode, DoubleScientificMode };
@@ -249,14 +257,14 @@ public:
         if (qIsInf(d))
             return float(d);
         if (std::fabs(d) > std::numeric_limits<float>::max()) {
-            if (ok != 0)
+            if (ok)
                 *ok = false;
             const float huge = std::numeric_limits<float>::infinity();
             return d < 0 ? -huge : huge;
         }
         if (d != 0 && float(d) == 0) {
             // Values that underflow double already failed. Match them:
-            if (ok != 0)
+            if (ok)
                 *ok = false;
             return 0;
         }
@@ -267,7 +275,6 @@ public:
     qint64 stringToLongLong(QStringView str, int base, bool *ok, QLocale::NumberOptions options) const;
     quint64 stringToUnsLongLong(QStringView str, int base, bool *ok, QLocale::NumberOptions options) const;
 
-    static double bytearrayToDouble(const char *num, bool *ok);
     // this function is used in QIntValidator (QtGui)
     Q_CORE_EXPORT static qint64 bytearrayToLongLong(const char *num, int base, bool *ok);
     static quint64 bytearrayToUnsLongLong(const char *num, int base, bool *ok);
@@ -284,9 +291,9 @@ public:
     quint16 m_language_id, m_script_id, m_country_id;
 
     // FIXME QTBUG-69324: not all unicode code-points map to single-token UTF-16 :-(
-    quint16 m_decimal, m_group, m_list, m_percent, m_zero, m_minus, m_plus, m_exponential;
-    quint16 m_quotation_start, m_quotation_end;
-    quint16 m_alternate_quotation_start, m_alternate_quotation_end;
+    char16_t m_decimal, m_group, m_list, m_percent, m_zero, m_minus, m_plus, m_exponential;
+    char16_t m_quotation_start, m_quotation_end;
+    char16_t m_alternate_quotation_start, m_alternate_quotation_end;
 
     quint16 m_list_pattern_part_start_idx, m_list_pattern_part_start_size;
     quint16 m_list_pattern_part_mid_idx, m_list_pattern_part_mid_size;
@@ -296,12 +303,6 @@ public:
     quint16 m_long_date_format_idx, m_long_date_format_size;
     quint16 m_short_time_format_idx, m_short_time_format_size;
     quint16 m_long_time_format_idx, m_long_time_format_size;
-    quint16 m_standalone_short_month_names_idx, m_standalone_short_month_names_size;
-    quint16 m_standalone_long_month_names_idx, m_standalone_long_month_names_size;
-    quint16 m_standalone_narrow_month_names_idx, m_standalone_narrow_month_names_size;
-    quint16 m_short_month_names_idx, m_short_month_names_size;
-    quint16 m_long_month_names_idx, m_long_month_names_size;
-    quint16 m_narrow_month_names_idx, m_narrow_month_names_size;
     quint16 m_standalone_short_day_names_idx, m_standalone_short_day_names_size;
     quint16 m_standalone_long_day_names_idx, m_standalone_long_day_names_size;
     quint16 m_standalone_narrow_day_names_idx, m_standalone_narrow_day_names_size;
@@ -327,22 +328,23 @@ public:
     quint16 m_weekend_end : 3;
 };
 
-class Q_CORE_EXPORT QLocalePrivate
+class Q_CORE_EXPORT QLocalePrivate // A POD type
 {
 public:
     static QLocalePrivate *create(
-            const QLocaleData *data,
+            const QLocaleData *data, const uint data_offset = 0,
             QLocale::NumberOptions numberOptions = QLocale::DefaultNumberOptions)
     {
-        QLocalePrivate *retval = new QLocalePrivate;
+        auto *retval = new QLocalePrivate;
         retval->m_data = data;
-        retval->ref.store(0);
+        retval->ref.storeRelaxed(0);
+        retval->m_data_offset = data_offset;
         retval->m_numberOptions = numberOptions;
         return retval;
     }
 
     static QLocalePrivate *get(QLocale &l) { return l.d; }
-    static const    QLocalePrivate *get(const QLocale &l) { return l.d; }
+    static const QLocalePrivate *get(const QLocale &l) { return l.d; }
 
     QChar decimal() const { return QChar(m_data->m_decimal); }
     QChar group() const { return QChar(m_data->m_group); }
@@ -357,28 +359,26 @@ public:
     quint16 countryId() const { return m_data->m_country_id; }
 
     QByteArray bcp47Name(char separator = '-') const;
+    QByteArray rawName(char separator = '-') const;
 
-    inline QLatin1String languageCode() const { return QLocalePrivate::languageToCode(QLocale::Language(m_data->m_language_id)); }
-    inline QLatin1String scriptCode() const { return QLocalePrivate::scriptToCode(QLocale::Script(m_data->m_script_id)); }
-    inline QLatin1String countryCode() const { return QLocalePrivate::countryToCode(QLocale::Country(m_data->m_country_id)); }
+    inline QLatin1String languageCode() const { return languageToCode(QLocale::Language(m_data->m_language_id)); }
+    inline QLatin1String scriptCode() const { return scriptToCode(QLocale::Script(m_data->m_script_id)); }
+    inline QLatin1String countryCode() const { return countryToCode(QLocale::Country(m_data->m_country_id)); }
 
     static QLatin1String languageToCode(QLocale::Language language);
     static QLatin1String scriptToCode(QLocale::Script script);
     static QLatin1String countryToCode(QLocale::Country country);
-    static QLocale::Language codeToLanguage(QStringView code) Q_DECL_NOTHROW;
-    static QLocale::Script codeToScript(QStringView code) Q_DECL_NOTHROW;
-    static QLocale::Country codeToCountry(QStringView code) Q_DECL_NOTHROW;
+    static QLocale::Language codeToLanguage(QStringView code) noexcept;
+    static QLocale::Script codeToScript(QStringView code) noexcept;
+    static QLocale::Country codeToCountry(QStringView code) noexcept;
     static void getLangAndCountry(const QString &name, QLocale::Language &lang,
                                   QLocale::Script &script, QLocale::Country &cntry);
 
     QLocale::MeasurementSystem measurementSystem() const;
 
-    QString dateTimeToString(QStringView format, const QDateTime &datetime,
-                             const QDate &dateOnly, const QTime &timeOnly,
-                             const QLocale *q) const;
-
     const QLocaleData *m_data;
     QBasicAtomicInt ref;
+    uint m_data_offset;
     QLocale::NumberOptions m_numberOptions;
 };
 
@@ -391,7 +391,7 @@ inline QLocalePrivate *QSharedDataPointer<QLocalePrivate>::clone()
 {
     // cannot use QLocalePrivate's copy constructor
     // since it is deleted in C++11
-    return QLocalePrivate::create(d->m_data, d->m_numberOptions);
+    return QLocalePrivate::create(d->m_data, d->m_data_offset, d->m_numberOptions);
 }
 
 inline char QLocaleData::digitToCLocale(QChar in) const
@@ -416,7 +416,7 @@ inline char QLocaleData::digitToCLocale(QChar in) const
     if (in == m_group)
         return ',';
 
-    if (in == m_exponential || in == QChar::toUpper(m_exponential))
+    if (in == m_exponential || in == QChar(QChar::toUpper(m_exponential)))
         return 'e';
 
     // In several languages group() is a non-breaking space (U+00A0) or its thin
